@@ -1,6 +1,6 @@
 /*
  * This file is Copyright (c) 2010 by the GPSD project
- * BSD terms apply: see the file COPYING in the distribution root for details.
+ * SPDX-License-Identifier: BSD-2-clause
  */
 
 /* cfmakeraw() needs _DEFAULT_SOURCE */
@@ -132,7 +132,7 @@ static int fusercount(const char *path)
 {
     DIR *procd, *fdd;
     struct dirent *procentry, *fdentry;
-    char procpath[32], fdpath[64], linkpath[64];
+    char procpath[64], fdpath[64], linkpath[64];
     int cnt = 0;
 
     if ((procd = opendir("/proc")) == NULL)
@@ -140,8 +140,9 @@ static int fusercount(const char *path)
     while ((procentry = readdir(procd)) != NULL) {
 	if (isdigit(procentry->d_name[0])==0)
 	    continue;
+        /* longest procentry->d_name I could find was 12 */
 	(void)snprintf(procpath, sizeof(procpath),
-		       "/proc/%s/fd/", procentry->d_name);
+		       "/proc/%.20s/fd/", procentry->d_name);
 	if ((fdd = opendir(procpath)) == NULL)
 	    continue;
 	while ((fdentry = readdir(fdd)) != NULL) {
@@ -727,7 +728,8 @@ void gpsd_close(struct gps_device_t *session)
 #ifdef TIOCNXCL
 	(void)ioctl(session->gpsdata.gps_fd, (unsigned long)TIOCNXCL);
 #endif /* TIOCNXCL */
-	(void)tcdrain(session->gpsdata.gps_fd);
+	if (!session->context->readonly)
+		(void)tcdrain(session->gpsdata.gps_fd);
 	if (isatty(session->gpsdata.gps_fd) != 0) {
 	    /* force hangup on close on systems that don't do HUPCL properly */
 	    (void)cfsetispeed(&session->ttyset, (speed_t) B0);
