@@ -1,11 +1,10 @@
-/* json.c - unit test for JSON partsing into fixed-extent structures
+/* json.c - unit test for JSON parsing into fixed-extent structures
  *
  * This file is Copyright (c) 2010 by the GPSD project
- * BSD terms apply: see the file COPYING in the distribution root for details.
+ * SPDX-License-Identifier: BSD-2-clause
  */
 
 #include <stdio.h>
-#include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stddef.h>
@@ -13,13 +12,22 @@
 
 #include "gpsd.h"
 #include "gps_json.h"
+#include "revision.h"
 
-#define JSON_MINIMAL	/* GPSD only uses a subset of the features */
+/* GPSD is built with JSON_MINIMAL.  Any !JSON_MINIMAL tests,
+ * like 18, 19 and 20 will thus fail.
+ * So this define removes them, they never execute.
+ */
+#define JSON_MINIMAL
 
-static void assert_case(int num, int status)
+static int debug = 0;
+static int current_test = 0;
+
+static void assert_case(int status)
 {
     if (status != 0) {
-	(void)fprintf(stderr, "case %d FAILED, status %d (%s).\n", num,
+	(void)fprintf(stderr, "case %d FAILED\n", current_test);
+	(void)fprintf(stderr, "status %d (%s).\n",
 		      status, json_error_string(status));
 	exit(EXIT_FAILURE);
     }
@@ -28,6 +36,7 @@ static void assert_case(int num, int status)
 static void assert_string(char *attr, char *fld, char *val)
 {
     if (strcmp(fld, val)) {
+	(void)fprintf(stderr, "case %d FAILED\n", current_test);
 	(void)fprintf(stderr,
 		      "'%s' string attribute eval failed, value = %s.\n",
 		      attr, fld);
@@ -38,6 +47,7 @@ static void assert_string(char *attr, char *fld, char *val)
 static void assert_integer(char *attr, int fld, int val)
 {
     if (fld != val) {
+	(void)fprintf(stderr, "case %d FAILED\n", current_test);
 	(void)fprintf(stderr,
 		      "'%s' integer attribute eval failed, value = %d.\n",
 		      attr, fld);
@@ -48,6 +58,7 @@ static void assert_integer(char *attr, int fld, int val)
 static void assert_uinteger(char *attr, unsigned int fld, unsigned int val)
 {
     if (fld != val) {
+	(void)fprintf(stderr, "case %d FAILED\n", current_test);
 	(void)fprintf(stderr,
 		      "'%s' integer attribute eval failed, value = %u.\n",
 		      attr, fld);
@@ -58,6 +69,7 @@ static void assert_uinteger(char *attr, unsigned int fld, unsigned int val)
 static void assert_boolean(char *attr, bool fld, bool val)
 {
     if (fld != val) {
+	(void)fprintf(stderr, "case %d FAILED\n", current_test);
 	(void)fprintf(stderr,
 		      "'%s' boolean attribute eval failed, value = %s.\n",
 		      attr, fld ? "true" : "false");
@@ -65,6 +77,15 @@ static void assert_boolean(char *attr, bool fld, bool val)
     }
 }
 
+static void assert_other(char *desc, int val, int val1)
+{
+    if (val != val1) {
+	(void)fprintf(stderr, "case %d FAILED\n", current_test);
+	(void)fprintf(stderr, "'%s' was %d, s/b %d\n", desc, val, val1);
+	exit(EXIT_FAILURE);
+    }
+}
+
 /*
  * Floating point comparisons are iffy, but at least if any of these fail
  * the output will make it clear whether it was a precision issue
@@ -72,6 +93,7 @@ static void assert_boolean(char *attr, bool fld, bool val)
 static void assert_real(char *attr, double fld, double val)
 {
     if (fld != val) {
+	(void)fprintf(stderr, "case %d FAILED\n", current_test);
 	(void)fprintf(stderr,
 		      "'%s' real attribute eval failed, value = %f.\n", attr,
 		      fld);
@@ -229,8 +251,37 @@ static const char *json_strOSC = "{\"class\":\"OSC\",\"device\":\"GPS#1\"," \
     "\"running\":true,\"reference\":true,\"disciplined\":false," \
     "\"delta\":67}";
 
+/* Case 13: test parsing of ERROR message, and some escape sequences */
+
+static char *json_strErr = "{\"class\":\"ERROR\",\"message\":" \
+                           "\"Hello\b\f\n\r\t\"}";
+
+/* Case 14: test parsing of ERROR message and \u escape */
+/* per ECMA-404, \u must be followed by 4 hex digits */
+
+static char *json_strErr1 = "{\"class\":\"ERROR\",\"message\":\"0\\u00334\"}";
+
+/* Case 15: test buffer overflow of short string destination */
+
+static char *json_strOver = "{\"name\":\"\\u0033\\u0034\\u0035\\u0036\"}";
+
+char json_short_string_dst[2];
+int json_short_string_cnt = 5;
+static const struct json_attr_t json_short_string[] = {
+    {"name", t_string,
+        .addr.string = json_short_string_dst,
+        .len = sizeof(json_short_string_dst)},
+    {"count", t_integer, .addr.integer = &json_short_string_cnt},
+    {NULL},
+};
+
+/* Case 16: test buffer overflow of short string destination */
+
+static char json_strOver2[7 * JSON_VAL_MAX];  /* dynamically built */
+
+
 #ifndef JSON_MINIMAL
-/* Case 13: Read array of integers */
+/* Case 17: Read array of integers */
 
 static const char *json_strInt = "[23,-17,5]";
 static int intstore[4], intcount;
@@ -242,7 +293,7 @@ static const struct json_array_t json_array_Int = {
     .maxlen = sizeof(intstore)/sizeof(intstore[0]),
 };
 
-/* Case 14: Read array of booleans */
+/* Case 18: Read array of booleans */
 
 static const char *json_strBool = "[true,false,true]";
 static bool boolstore[4];
@@ -255,13 +306,13 @@ static const struct json_array_t json_array_Bool = {
     .maxlen = sizeof(boolstore)/sizeof(boolstore[0]),
 };
 
-/* Case 15: Read array of reals */
+/* Case 19: Read array of reals */
 
-static const char *json_str15 = "[23.1,-17.2,5.3]";
+static const char *json_strReal = "[23.1,-17.2,5.3]";
 static double realstore[4];
 static int realcount;
 
-static const struct json_array_t json_array_15 = {
+static const struct json_array_t json_array_Real = {
     .element_type = t_real,
     .arr.reals.store = realstore,
     .count = &realcount,
@@ -273,13 +324,22 @@ static const struct json_array_t json_array_15 = {
 
 static void jsontest(int i)
 {
-    int status = 0;
+    int status = 0;   /* libgps_json_unpack() returned status */
+    int n;            /* generic index */
+
+    if (0 < debug) {
+	(void)fprintf(stderr, "Running test #%d.\n", i);
+    }
+    current_test = i;
+
+    /* do not keep old data! */
+    memset((void *)&gpsdata, 0, sizeof(gpsdata));
 
     switch (i)
     {
     case 1:
 	status = libgps_json_unpack(json_str1, &gpsdata, NULL);
-	assert_case(1, status);
+	assert_case(status);
 	assert_string("device", gpsdata.dev.path, "GPS#1");
 	assert_integer("mode", gpsdata.fix.mode, 3);
 	assert_real("time", gpsdata.fix.time, 1119168761.8900001);
@@ -289,7 +349,7 @@ static void jsontest(int i)
 
     case 2:
 	status = libgps_json_unpack(json_str2, &gpsdata, NULL);
-	assert_case(2, status);
+	assert_case(status);
 	assert_integer("used", gpsdata.satellites_used, 6);
 	assert_integer("PRN[0]", gpsdata.skyview[0].PRN, 10);
 	assert_integer("el[0]", gpsdata.skyview[0].elevation, 45);
@@ -305,26 +365,26 @@ static void jsontest(int i)
 
     case 3:
 	status = json_read_array(json_str3, &json_array_3, NULL);
-	assert_case(3, status);
-	assert(stringcount == 3);
-	assert(strcmp(stringptrs[0], "foo") == 0);
-	assert(strcmp(stringptrs[1], "bar") == 0);
-	assert(strcmp(stringptrs[2], "baz") == 0);
+	assert_case(status);
+	assert_other("stringcount", stringcount, 3);
+	assert_other("stringptrs[0] == foo", strcmp(stringptrs[0], "foo"), 0);
+	assert_other("stringptrs[1] == bar", strcmp(stringptrs[1], "bar"), 0);
+	assert_other("stringptrs[2] == baz", strcmp(stringptrs[2], "baz"), 0);
 	break;
 
     case 4:
 	status = json_read_object(json_str4, json_attrs_4, NULL);
-	assert_case(4, status);
-	assert_integer("dftint", dftinteger, -5);	/* did the default work? */
-	assert_uinteger("dftuint", dftuinteger, 10);	/* did the default work? */
-	assert_real("dftreal", dftreal, 23.17);	/* did the default work? */
+	assert_case(status);
+	assert_integer("dftint", dftinteger, -5);    /* did the default work? */
+	assert_uinteger("dftuint", dftuinteger, 10); /* did the default work? */
+	assert_real("dftreal", dftreal, 23.17);	     /* did the default work? */
 	assert_boolean("flag1", flag1, true);
 	assert_boolean("flag2", flag2, false);
 	break;
 
     case 5:
 	status = libgps_json_unpack(json_str5, &gpsdata, NULL);
-	assert_case(5, status);
+	assert_case(status);
 	assert_string("path", gpsdata.dev.path, "/dev/ttyUSB0");
 	assert_integer("flags", gpsdata.dev.flags, 5);
 	assert_string("driver", gpsdata.dev.driver, "Foonly");
@@ -332,7 +392,7 @@ static void jsontest(int i)
 
     case 6:
 	status = json_read_object(json_str6, json_attrs_6, NULL);
-	assert_case(6, status);
+	assert_case(status);
 	assert_integer("dumbcount", dumbcount, 4);
 	assert_string("dumbstruck[0].name", dumbstruck[0].name, "Urgle");
 	assert_string("dumbstruck[1].name", dumbstruck[1].name, "Burgle");
@@ -350,7 +410,7 @@ static void jsontest(int i)
 
     case 7:
 	status = libgps_json_unpack(json_str7, &gpsdata, NULL);
-	assert_case(7, status);
+	assert_case(status);
 	assert_string("release", gpsdata.version.release, "2.40dev");
 	assert_string("rev", gpsdata.version.rev, "dummy-revision");
 	assert_integer("proto_major", gpsdata.version.proto_major, 3);
@@ -359,7 +419,7 @@ static void jsontest(int i)
 
     case 8:
 	status = json_read_object(json_str8, json_attrs_8, NULL);
-	assert_case(8, status);
+	assert_case(status);
 	assert_integer("fee", fee, 3);
 	assert_integer("fie", fie, 6);
 	assert_integer("foe", foe, 14);
@@ -368,13 +428,13 @@ static void jsontest(int i)
     case 9:
 	/* yes, the '6' in the next line is correct */
 	status = json_read_object(json_str9, json_attrs_6, NULL);
-	assert_case(9, status);
+	assert_case(status);
 	assert_integer("dumbcount", dumbcount, 0);
 	break;
 
     case 10:
 	status = json_pps_read(json_strPPS, &gpsdata, NULL);
-	assert_case(10, status);
+	assert_case(status);
 	assert_string("device", gpsdata.dev.path, "GPS#1");
 	assert_integer("real_sec", gpsdata.pps.real.tv_sec, 1428001514);
 	assert_integer("real_nsec", gpsdata.pps.real.tv_nsec, 1000000);
@@ -384,7 +444,7 @@ static void jsontest(int i)
 
     case 11:
 	status = json_toff_read(json_strTOFF, &gpsdata, NULL);
-	assert_case(11, status);
+	assert_case(status);
 	assert_string("device", gpsdata.dev.path, "GPS#1");
 	assert_integer("real_sec", gpsdata.pps.real.tv_sec, 1428001514);
 	assert_integer("real_nsec", gpsdata.pps.real.tv_nsec, 1000000);
@@ -394,7 +454,7 @@ static void jsontest(int i)
 
     case 12:
 	status = json_oscillator_read(json_strOSC, &gpsdata, NULL);
-	assert_case(12,status);
+	assert_case(status);
 	assert_string("device", gpsdata.dev.path, "GPS#1");
 	assert_boolean("running", gpsdata.osc.running, true);
 	assert_boolean("reference", gpsdata.osc.reference, true);
@@ -402,10 +462,80 @@ static void jsontest(int i)
 	assert_integer("delta", gpsdata.osc.delta, 67);
 	break;
 
-#ifdef JSON_MINIMAL
-#define MAXTEST 12
-#else
     case 13:
+	if (2 < debug) {
+	    (void)fprintf(stderr, "test string: %s.\n", json_strErr);
+	}
+	status = libgps_json_unpack(json_strErr, &gpsdata, NULL);
+	assert_case(status);
+	assert_string("message", gpsdata.error, "Hello\b\f\n\r\t");
+	break;
+
+    case 14:
+	if (2 < debug) {
+	    (void)fprintf(stderr, "test string: %s.\n", json_strErr1);
+	}
+	status = libgps_json_unpack(json_strErr1, &gpsdata, NULL);
+	assert_case(status);
+	assert_string("message", gpsdata.error, "034");
+	break;
+
+    case 15:
+        /* check for string overrun caught */
+	if (2 < debug) {
+	    (void)fprintf(stderr, "test string: %s.\n", json_strOver);
+	}
+	json_short_string_cnt = 7;
+	status = json_read_object(json_strOver, json_short_string, NULL);
+	assert_case(JSON_ERR_STRLONG != status);
+	assert_string("name", json_short_string_dst, "");
+	assert_integer("count", json_short_string_cnt, 0);
+	break;
+
+    case 16:
+        /* check for string overrun caught */
+        json_strOver2[0] = '\0';
+        /* build a LONG test string */
+	strlcat(json_strOver2, "{\"name\":\"", sizeof(json_strOver2));
+        for (n = 0; n < (2 * JSON_VAL_MAX); n++) {
+	    strlcat(json_strOver2, "\\u0033", sizeof(json_strOver2));
+        }
+	strlcat(json_strOver2, "\"}", sizeof(json_strOver2));
+
+	if (2 < debug) {
+	    (void)fprintf(stderr, "test string: %s.\n", json_strOver2);
+	}
+	json_short_string_cnt = 7;
+	status = json_read_object(json_strOver2, json_short_string, NULL);
+	assert_case(JSON_ERR_STRLONG != status);
+	assert_string("name", json_short_string_dst, "");
+	assert_integer("count", json_short_string_cnt, 0);
+	break;
+
+    case 17:
+        /* check for a different string overrun caught */
+        json_strOver2[0] = '\0';
+        /* build a LONG test string */
+	strlcat(json_strOver2, "{\"name\":\"", sizeof(json_strOver2));
+        for (n = 0; n < (2 * JSON_VAL_MAX); n++) {
+	    strlcat(json_strOver2, "\\A", sizeof(json_strOver2));
+        }
+	strlcat(json_strOver2, "\"}", sizeof(json_strOver2));
+
+	if (2 < debug) {
+	    (void)fprintf(stderr, "test string: %s.\n", json_strOver2);
+	}
+	json_short_string_cnt = 7;
+	status = json_read_object(json_strOver2, json_short_string, NULL);
+	assert_case(JSON_ERR_STRLONG != status);
+	assert_string("name", json_short_string_dst, "");
+	assert_integer("count", json_short_string_cnt, 0);
+	break;
+
+#ifdef JSON_MINIMAL
+#define MAXTEST 17
+#else
+    case 18:
 	status = json_read_array(json_strInt, &json_array_Int, NULL);
 	assert_integer("count", intcount, 3);
 	assert_integer("intstore[0]", intstore[0], 23);
@@ -414,7 +544,7 @@ static void jsontest(int i)
 	assert_integer("intstore[3]", intstore[3], 0);
 	break;
 
-    case 14:
+    case 19:
 	status = json_read_array(json_strBool, &json_array_Bool, NULL);
 	assert_integer("count", boolcount, 3);
 	assert_boolean("boolstore[0]", boolstore[0], true);
@@ -423,8 +553,8 @@ static void jsontest(int i)
 	assert_boolean("boolstore[3]", boolstore[3], false);
 	break;
 
-    case 15:
-	status = json_read_array(json_str15, &json_array_15, NULL);
+    case 20:
+	status = json_read_array(json_strReal, &json_array_Real, NULL);
 	assert_integer("count", realcount, 3);
 	assert_real("realstore[0]", realstore[0], 23.1);
 	assert_real("realstore[1]", realstore[1], -17.2);
@@ -432,7 +562,7 @@ static void jsontest(int i)
 	assert_real("realstore[3]", realstore[3], 0);
 	break;
 
-#define MAXTEST 15
+#define MAXTEST 20
 #endif /* JSON_MINIMAL */
 
     default:
@@ -446,11 +576,12 @@ int main(int argc UNUSED, char *argv[]UNUSED)
     int option;
     int individual = 0;
 
-    while ((option = getopt(argc, argv, "hn:D:?")) != -1) {
+    while ((option = getopt(argc, argv, "D:hn:V?")) != -1) {
 	switch (option) {
 #ifdef CLIENTDEBUG_ENABLE
 	case 'D':
-	    gps_enable_debug(atoi(optarg), stdout);
+	    debug = atoi(optarg);
+	    gps_enable_debug(debug, stdout);
 	    break;
 #endif
 	case 'n':
@@ -459,19 +590,29 @@ int main(int argc UNUSED, char *argv[]UNUSED)
 	case '?':
 	case 'h':
 	default:
-	    (void)fputs("usage: test_json [-D lvl]\n", stderr);
+	    (void)fprintf(stderr,
+                        "usage: %s [-D lvl] [-n tst] [-V]\n"
+                        "       -D lvl      set debug level\n"
+                        "       -n tst      run only test tst\n"
+                        "       -V          Print version and exit\n",
+                        argv[0]);
 	    exit(EXIT_FAILURE);
+        case 'V':
+            (void)fprintf(stderr, "%s: %s (revision %s)\n",
+                          argv[0], VERSION, REVISION);
+            exit(EXIT_SUCCESS);
 	}
     }
 
-    (void)fprintf(stderr, "JSON unit test ");
+    (void)fprintf(stderr, "JSON unit tests\n");
 
     if (individual)
 	jsontest(individual);
     else {
 	int i;
-	for (i = 1; i <= MAXTEST; i++)
+	for (i = 1; i <= MAXTEST; i++) {
 	    jsontest(i);
+        }
     }
 
     (void)fprintf(stderr, "succeeded.\n");
